#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'1.00',
 'build_date': '2010-01-28 10:13:13 -0800',
 'clean': None,
 'date': '2010-01-27 17:44:36 -0800',
 'revision_id': 'root@peacemaker-20100128014436-gbd9bn8vgy2n81u5',
 'revno': 721}

revisions = {}

file_revisions = {}



if __name__ == '__main__':
    print 'revision: %(revno)d' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info
